;-----------------------------------------------------
;Gestione routine comunicazione

;Permette di leggere o scrivere i settaggi utente sulla Eeprom della cpu.
;Si entra in questa routine se l'utente sposta l'apposito deviatore in posizione
;'Comunicazione ON' (viene a mancare il segnale di zero crossing dalla rete).
;Per uscire da questa routine l'utente deve spostare lo stesso deviatore su
;'Comunicazione OFF' e resettare la cpu di conseguenza.
;
;Poich  possibile ricevere un solo carattere alla volta, per gestire le
;funzioni desiderate sono stati definiti due stati di comunicazione:

; 1) Attesa comandi: sono accettati i seguenti caratteri
;       Carattere Rx    Risposta                Funzione
;       ------------    --------                --------
;       R               Raabbcc..<CrLf>         Lettura dati da EEprom in formato hex aa
;       W               W<CrLf>                 Entra nel loop scrittura dati
;       altri           E<CrLf>                 Comando sconosciuto

; 2) Loop scrittura dati: il carattere pervenuto (in binario) viene scritto sulla
;    locazione Eeprom corrente. Dopo di ci, il dato viene riletto e si trasmette
;    indietro il suo valore esadecimale per informare sullo stato della scrittura
;    precedentemente effettuata (e per consentire la trasmissione del
;    prossimo dato da scrivere).
;    Per scrivere su tutte le locazioni interessate  necessario inviare altrettanti
;    caratteri dopo ogni risposta di consenso.
;    Si esce dal loop di scrittura dati se sono state scritte tutte le locazioni
;    oppure se non giunge un carattere (da scrivere) entro 2 secondi circa.
;    L'esempio mostra una procedura di scrittura

;       Carattere Rx    Risposta                Note
;       ------------    --------                ----
;       W               W<CrLf>                 Entra nel loop di scrittura
;       Ascii 160       WA0<CrLf>               Scrive 160 (0xA0) all'indirizzo 0
;       Ascii 1         W01<CrLf>               Scrive 001 (0x01) all'indirizzo 1
;       Ascii 67        W43<CrLf>               Scrive 067 (0x43) all'indirizzo 2
;       ....
;       Ascii 15        W0F<CrLf>               Scrive 015 (0x0F) all'indirizzo n

; Usate: Count1, Count2, RegTxRx
; Usate: CntDel1, CntDel2, CntDel3

make_comm
        call    out_irq_off     ;Disattiva irq e spegne tutte le uscite
        bsf     O_TX            ;Attiva l'uscita tx (invertita)

mc_0    clrf    Count1          ;Esce dal loop di scrittura (se Count1<>0)

mc_1    movlw   D'6'            ;Ritardo massimo per attendere prossimo
        movwf   CntDel3         ;carattere nel loop di scrittura
        clrf    CntDel2
        clrf    CntDel1

mc_2    btfss   I_RX            ;Aspetta il fronte 1-->0 del bit di Start
        goto    mc_3            ;di un carattere da ricevere

        clrwdt
        movf    Count1,w        ;Ritardo per uscire dal loop di scrittura
        btfsc   STATUS,Z        ;(Count1<>0) se non giungono comandi entro 2s circa
        goto    mc_2
        decfsz  CntDel1,f
        goto    mc_2
        decfsz  CntDel2,f
        goto    mc_2
        decfsz  CntDel3,f
        goto    mc_2
        goto    mc_0

mc_3    call    rx_char         ;Riceve il carattere dalla porta seriale

        movf    Count1,w        ;Se Count1<>0 salta a loop di scrittura
        btfss   STATUS,Z
        goto    mc_write_loop

        movf    RegTxRx,w       ;Se ricevuto comando 'R', salta a lettura EEprom
        xorlw   'R'
        btfsc   STATUS,Z
        goto    mc_read

        movf    RegTxRx,w       ;Se ricevuto comando 'W', entra nel loop di scrittura
        xorlw   'W'
        btfsc   STATUS,Z
        goto    mc_write_start

        movlw   'E'             ;Se carattere sconosciuto, invia segnalazione
        call    tx_char         ;di errore
        call    tx_crlf
        goto    mc_1


; Comando per lettura Eeprom
mc_read
        movlw   'R'             ;Invia carattere R
        call    tx_char

        clrf    Count2          ;Trasmette tutti i dati contenuti nella Eeprom
mcr_1   movf    Count2,w        ;in formato esadecimale
        call    read_eeprom
        call    print_hex
        incf    Count2,f        ;Incrementa indirizzo e lo confronta con il massimo
        movf    Count2,w
        xorlw   MAXEEA
        btfss   STATUS,Z
        goto    mcr_1

        call    tx_crlf         ;Terminatore Cr+Lf
        goto    mc_1            ;Ritorna in attesa comando


; Comando per iniziare il loop di scrittura su Eeprom
mc_write_start
        movlw   'W'             ;Invia W<CrLf>
        call    tx_char
        call    tx_crlf
        movlw   0x01            ;Inizializza contatore per indirizzo Eeprom
        movwf   Count1
        goto    mc_1            ;Ritorna in loop scrittura


; Loop per scrittura carattere su Eeprom
mc_write_loop
        decf    Count1,w        ;Indirizzo Eeprom = Count1-1
        BANK1
        movwf   EEADR
        BANK0
        movf    RegTxRx,w       ;Dato ricevuto da scrivere
        BANK1
        movwf   EEDATA
;       BANK0
        call    write_eeprom


        movlw   'W'             ;Legge il dato appena scritto e lo ritrasmette
        call    tx_char         ;nel formato Whh<CrLf>, dove
        decf    Count1,w        ;hh sono le due cifre esadecimali del valore
        call    read_eeprom
        call    print_hex
        call    tx_crlf

        movf    Count1,w        ;Incrementa contatore per prossimo indirizzo da
        xorlw   MAXEEA          ;scrivere su Eeprom (se <> dal massimo)
        btfsc   STATUS,Z
        goto    mc_0            ;Fine loop di scrittura
        incf    Count1,f
        goto    mc_1            ;Attende prossimo carattere da scrivere


;******************************************************************
; MACRO - Ritardo per trasmissione/ricezione su porta seriale
; Ritardo ottenuto = ((VALDELBIT-1)*4+5)*(1/(Fosc/4))
; Esempio: VALDELBIT=48 e Fosc=8MHz --> Delay = 96.5us
; Input:  VALDELBIT
; Usate:  CDelBit

DELBIT  MACRO   VALDELBIT
        LOCAL   del_b1

        movlw   VALDELBIT
        movwf   CDelBit
del_b1  clrwdt                  ;Azzera watch dog
        decfsz  CDelBit,f
        goto    del_b1

        ENDM		    	;Fine macro


;-----------------------------------------------------
; Invia un carattere sulla porta seriale alla velocit
; 9600 baud, 8 bit dati, 1 bit stop, No parit (1bit = 104us)
; Non si utilizza la USART presente a bordo della cpu per 
; mantenere la compatibilit hardware.

; Input:  w             Carattere da inviare
; Output: O_TX          Uscita trasmissione seriale
; Usate:  RegTxRx       Appoggio per trasmissione
; Usate:  CBit          Contatore bit

tx_char	movwf   RegTxRx		;RegTxRx = w
        movlw   0x08           	;8 bit da trasmettere
        movwf   CBit
        bcf	O_TX       	;Bit di Start
        DELBIT  D'51'	  	;Attesa 102.5 us
txch_1  btfss   RegTxRx,0	;Loop Tx
        goto    txch_2
        nop
        bsf     O_TX
        goto    txch_3
txch_2  bcf     O_TX
        goto    txch_3
txch_3  nop
        rrf     RegTxRx,f
        DELBIT  D'49'		;Attesa 98.5 us
        nop
        decfsz  CBit,f
        goto    txch_1
        bsf     O_TX       	;Bit di Stop
        DELBIT  D'54'		;Attesa 108.5 us
        return


;-----------------------------------------------------
; Riceve il carattere dalla porta seriale alla velocit di
; 9600 baud, 8 bit dati, 1 stop bit, No parit (1bit = 104us)
; Non si utilizza la USART presente a bordo della cpu per 
; mantenere la compatibilit hardware.

; Input:  I_RX          Ingresso ricezione seriale
; Output: RegTxRx       Carattere ricevuto
; Usate:  CBit          Contatore bit

rx_char	clrf    RegTxRx
        movlw   0x08            ;8 bit da ricevere
        movwf   CBit
        DELBIT  D'77'  		;Attesa 1.5 bit (154.5 us)
rxch_1	btfss   I_RX	        ;Loop di lettura su porta seriale
        goto    rxch_2
        nop
        bsf     STATUS,C
        goto    rxch_3
rxch_2  bcf     STATUS,C
        goto    rxch_3
rxch_3  nop
        rrf     RegTxRx,f
        DELBIT  D'49'   	;Wait 1 bit (98.5 us)
        decfsz  CBit,f
        goto    rxch_1
        return


;-----------------------------------------------------
;Invia Cr e Lf sulla porta seriale
tx_crlf	movlw   D'13'
	call    tx_char
	movlw	D'10'
	call    tx_char
	return


;---------------------------------------------------------------------------------
;Stampa il byte contenuto in w nei corrispondenti 2 caratteri Ascii esadecimali
;Esempi: 1) w=32 --> Stampa='20'   2)w='A' (65) --> Stampa='41'

;Input:  w                      carattere da convertire e stampare
;Usati:  Value, Temp            appoggi temporanei

print_hex
	movwf	 Temp           ;Salva w in Temp

	andlw	 B'11110000'	;Converte e stampa cifra H
	movwf	 Value
	swapf	 Value,f
	movlw	 D'10'
	subwf	 Value,w        ;w = Value - 10
	btfss	 STATUS,C	;Verifica se >= 10 (numero 0..9 o lettera A..F)
	goto	 prh_1
	addlw	 'A'
	goto	 prh_2
prh_1	movf	 Value,w
	addlw	 '0'
prh_2	call     tx_char

	movf	 Temp,w	        ;Converte e stampa cifra L
	andlw	 B'00001111'
	movwf	 Value
	movlw	 D'10'
	subwf	 Value,w        ;w = Value - 10
	btfss	 STATUS,C	;Verifica se > 10 (numero 0..9 o lettera A..F)
	goto	 prh_3
	addlw	 'A'
	goto	 prh_4
prh_3	movf	 Value,w
	addlw	 '0'
prh_4	call     tx_char
	return

